package jnyqide;

import java.util.ArrayList;
import java.io.BufferedReader;

/***
** Class: InstrumentCharacteristics
** Author: Priyanka Raghavan and Roger B. Dannenberg
** Description: The instrument characteristics class reads from 
** the instruments.txt file and stores the
** instrument characteristics such as the subcategory,name,library.
** The characteristics include the implementation and parameter
** description. The different parameters are added as an arraylist.
**/

public class InstrumentCharacteristics {
    
    private String categoryName;
    private String subcategoryName;
    private String functionName;
    private ArrayList parameterList;
    private String implementation;

    private char buffer;
    private boolean bufferp;

    InstrumentCharacteristics() {
        bufferp = false;
	parameterList = new ArrayList();
    }

    public String getCategoryName() { return categoryName; }

    public void setCategoryName(String name) { categoryName = name; }

    public void setSubcategoryName(String subname) {
	subcategoryName = subname;
    }

    public String getSubcategoryName() { return subcategoryName; }

    public String getFunctionName() { return functionName; }


    public void addParameter(String name, String minValue, String maxValue, 
                             String defaultValue, String type) {
	Parameter parameter = new Parameter(name, minValue, maxValue, 
					    defaultValue, type);
	parameterList.add(parameter);
    }
    
    public ArrayList getParameters(){ return parameterList; }

    public String getImplementation() { return implementation; }

    public boolean readData(BufferedReader br) {
	categoryName = getToken(br); // category
	if (categoryName == null) return false;
	if (getNonSpace(br) != ':') {
	    System.out.println("expected : after " + categoryName);
	    return false;
	}
	subcategoryName = getToken(br);
	int c = getNonSpace(br);
	functionName = subcategoryName;
	if (c == '[') {
	    functionName = getToken(br);
	    if (getNonSpace(br) != ']') {
		System.out.println("expected ] after " + functionName);
		return false;
	    }
	} else ungetChar(c);
	if (getNonSpace(br) != '(') {
	    System.out.println("no ( after " + functionName);
	    return false;
	}
	while ((c = getNonSpace(br)) != ')') {
	    ungetChar(c);
	    Parameter p = readParameter(br);
	    if (p == null) {
		System.out.println("syntax error for parameter in " + 
				   subcategoryName);
		return false;
	    }
	    parameterList.add(p);
	}
	String line;
	try {
	    line = br.readLine();
	    while (line != null && line.length() < 3) {
		// skip blank lines -- we're not checking too carefully
		// but a char count of 3 allows only CRLF and maybe a
		// space or tab
		line = br.readLine();
	    }
	    if (line == null || line.indexOf("SOURCE-BEGIN") != 0) {
		System.out.println("expected SOURCE-BEGIN, not " + line);
		return false;
	    }
	    // read until SOURCE-END
	    implementation = "";
	    while ((line = br.readLine()) != null && 
		   line.indexOf("SOURCE-END") != 0) {
		implementation = implementation + line + "\n";
	    }
	    if (line == null || line.indexOf("SOURCE-END") != 0) {
		System.out.println("expected SOURCE-END, not " + line);
		return false;
	    }
	} catch (Exception e) {
	    return false;
	}	    
	return true;
    }

    private Parameter readParameter(BufferedReader br) {
	Parameter p = new Parameter();
	String tok = getToken(br);
	if (tok == null) {
	    System.out.println("expected parameter type: " + tok);
	    return null;
	}
	p.setType(tok);
	int param1 = getNonSpace(br);
	if (param1 != ':') {
	    ungetChar(param1);
	}
	tok = getToken(br);
	if (tok == null) {
	    System.out.println("expected parameter name: " + tok);
	    return null;
	}
	if (param1 == ':') tok = ":" + tok;
	p.setName(tok);

	if (getNonSpace(br) != '=') {
	    System.out.println("expected = after parameter: " + tok);
	    return null;
	}

	tok = getToken(br);
	if (tok == null) {
	    System.out.println("expected default value: " + tok);
	    return null;
	}
	p.setDefaultValue(tok);

	if (getNonSpace(br) != '(') {
	    System.out.println("expected ( after default value: " + tok);
	    return null;
	}
	
	tok = getToken(br);
	if (tok == null) {
	    System.out.println("expected min value: " + tok);
	    return null;
	}
	p.setMinValue(tok);

	if (getNonSpace(br) != ':') {
	    System.out.println("expected : after min value: " + tok);
	    return null;
	}
	
	tok = getToken(br);
	if (tok == null) {
	    System.out.println("expected max value: " + tok);
	    return null;
	}
	p.setMaxValue(tok);

	if (getNonSpace(br) != ')') {
	    System.out.println("expected ) after max value: " + tok);
	    return null;
	}
	
	int c = getNonSpace(br);
	if (c != ',') ungetChar(c);

	return p;
    }

    private int getNonSpace(BufferedReader br) {
        int c;
        while ((c = getChar(br)) != -1 && Character.isWhitespace(c));
	return c;
    }

    private int getChar(BufferedReader br) {
        if (bufferp) {
	    bufferp = false;
	    return buffer;
	}
	try {
	    return br.read();
	} catch (Exception e) {
	    return -1;
	}
    }

    private void ungetChar(int c) {
	if (c == -1) return; // ignore EOF
	buffer = (char) c;
	bufferp = true;
    }

    private String getToken(BufferedReader br) {
	int c = getNonSpace(br);
	StringBuffer token = new StringBuffer();
        while (c != -1 && (Character.isLetterOrDigit(c) || 
			   c == '-' || c == '.')) {
	    token.append((char) c);
	    c = getChar(br);
	}
	ungetChar(c);
	String s = new String(token);
	if (s.length() == 0) return null;
	// System.out.println("gettoken: " + token);
        return s;
    }
}


/**
 ** Class: Parameter
 ** Author: Priyanka Raghavan
 ** Description: This class is used to store parameter values like 
 **    name, minvalue, maxvalue, default value, and type (integer,string, etc.)
 **
 **/

class Parameter{
    String name;
    String minValue;
    String maxValue;
    String defaultValue;
    String type;
    float value;
    
    Parameter() { }

    Parameter(String name, String minValue, String maxValue,
	      String defaultValue, String type) {
	this.name = name;
	this.minValue = minValue;
	this.maxValue = maxValue;
	this.defaultValue = defaultValue;
	this.type=type;
	value = 0.0f;
    }

    public void setName(String name) {
	this.name=name;
    }

    public void setMinValue(String value) {
	minValue = value;
    }

    public void setMaxValue(String value) {
	maxValue = value;
    }

    public void setDefaultValue(String defaultvalue) {
	defaultValue = defaultvalue;
    }

    public String getName() {
	return name;
    }

    public String getMinValue() {
	return minValue;
    }

    public String getMaxValue() {
	return maxValue;
    }
    
    public String getType() { return type; }
    
    public void setType(String type) { this.type = type; }
    
    public String getDefaultValue() { return defaultValue; }

    public float getValue() { return value; }

    public void setValue(float f) { value = f; }

}


    /*
     * 
     *  (non-Javadoc)
     * @see java.lang.Runnable#run()
     */
/*
    public void populateCombos() {
	instruCategoryCombo.addItem("Select One");
	subcategoryCombo.addItem("Selet One");
	String line;
	String lineInput[]=null;
	try {
	    // read the corresponding instrument.txt text file from lib
	    // and create the InstrumentCharacteristic objects.
	    //
	    BufferedReader br = new BufferedReader(new FileReader(nyquistThread.soundBrowser));
	    while((line= br.readLine())!=null) {
        	
        	InstrumentCharacteristics new_instr=null;
        	
		// formatted item with spaces in between
		lineInput=line.split(" ");
		//
		// populate the combo boxes
		//
		for(int i=0;i<lineInput.length;i++) {
		    if (lineInput[i].equalsIgnoreCase("instrument")) {
			int check=0;
			String instrument=lineInput[i+1].toString();
			new_instr= new InstrumentCharacteristics(instrument);
			for(int k=0;k<instruCategoryCombo.getItemCount();k++) {
			    if (instruCategoryCombo.getItemAt(k).toString().equalsIgnoreCase(instrument)) {
				check=1;
				break;
			    }
			    
			}//end of for
			if (check==0)
			    instruCategoryCombo.addItem(instrument);
			
			
			
		    }//end fof if instrument
		    if (lineInput[i].equalsIgnoreCase("category")) {
			int check=0;
			String instrument=lineInput[i+1].toString();
			for(int k=0;k<subcategoryCombo.getItemCount();k++) {
			    if (subcategoryCombo.getItemAt(k).toString().equalsIgnoreCase(instrument)) {
				check=1;
				break;
			    }
			    
			}//end of for
			//Check for duplicates
			if (check==0)
			    subcategoryCombo.addItem(lineInput[i+1].toString());
			Instruments.add(new_instr);
			new_instr.setSubCategoryName(lineInput[i+1].toString());
			addAttributes(lineInput,new_instr);
			
			
			
		    }
		}//end of for
		
	    }//end of while
	    br.close();
	} catch (Exception e) {
	    
	}
	
    }
*/
    /*
     * @ desc This method is used to add various parameters for the instruments.
     * The library that needs to be loaded for the instrument is also set here.
     ****/
/*    void addAttributes(String[] lineInput, InstrumentCharacteristics new_instr) {
	int i=0;
	System.out.println("lineInput"+lineInput.length);
	System.out.println("Instrument"+new_instr.getCategoryName());
	System.out.println("Sub category"+new_instr.getSubcategoryName());
	while(i<lineInput.length) {
	    System.out.println("lineInput"+lineInput[i]);
	    String name=null;
	    String type=null;
	    String default_value=null;
	    String min_value=null;
	    String max_value=null;	
	    String library=null;
	    if (lineInput[i].equalsIgnoreCase("parameter")) {
		i++;
		name=lineInput[i];
		i=i+2;
		//This should be for type
		type= lineInput[i];
		i=i+2;
		default_value= lineInput[i];
		i++;
		
		min_value= lineInput[i+1];
		max_value= lineInput[i+2];
		i=i+2;
		
		System.out.println("Name:"+name +"type:"+ type +"default value:"+ default_value+"min value"+min_value+"max value"+max_value);
		System.out.println("Line input befor heck for lib"+lineInput[i]);
		if (lineInput[i].equalsIgnoreCase("load")) {
		    library= lineInput[i+1];
		    System.out.println("library in addparameter"+library);
		}
		else 
		    library="";
		new_instr.addParameter(name,min_value,max_value,default_value,type);
		
		System.out.println("Instru name" +
		                   new_instr.getCategoryName() + 
				   " Library"+new_instr.getLibrary());
	    }
	    if (lineInput[i].equalsIgnoreCase("load")) {
		library= lineInput[i+1];
		System.out.println("library in addparameter"+library);
		new_instr.setLibrary(library);
	    }
	    i++;	
	}
    }
*/
